local _G = _G;

if select(6, _G.GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end
local PitBull4 = _G.PitBull4;
if not PitBull4 then error("PitBull4_InlinePower requires PitBull4") end

--local L = PitBull4.L;

local changed = false;

local POWER = {
	enabled = true,
	reqlevel = 0,
	color_override = {},
	events = {"UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER", "PLAYER_ENTERING_WORLD"},
};

local module = PitBull4:NewModule("InlinePower", "AceEvent-3.0");

module:SetModuleType("bar");
module:SetName("Inline Power");
module:SetDescription("Replace for PitBulls power icon indicators of spec-specific powers.");
module:SetDefaults({
	position = 3
});

function module:OnEnable()
	for _,event in ipairs(POWER.events) do
		self:RegisterEvent(event);
	end
end

function module:OnDisable()
	self:UnregisterAllEvents();
end

local function update_player(self)
	for frame in PitBull4:IterateFramesForUnitID("player") do
		self:Update(frame);
	end
end

function module:PLAYER_ENTERING_WORLD()
	update_player(self);
end

function module:UNIT_POWER_FREQUENT(event, unit, power)
	if( unit == "player" and power == POWER.name ) then
		update_player(self);
	end
end

function module:UNIT_DISPLAYPOWER(event, unit)
	if( unit == "player" ) then
		update_player(self);
	end
end

function module:RUNE_POWER_UPDATE()
	update_player(self);
end

function module:RUNE_TYPE_UPDATE()
	update_player(self);
end

function module:PLAYER_TALENT_UPDATE()
	spec = _G.GetSpecialization();
	
	local new_value = spec == POWER.reqspec;
	if( new_value ~= POWER.enabled ) then
		changed = true;
	end
	
	POWER.enabled = new_value;
	update_player(self);
end

local mop_500 = select(4, _G.GetBuildInfo()) >= 50000;

local class = select(2, _G.UnitClass("player"));
local level = _G.UnitLevel("player");
local spec = _G.GetSpecialization();

if( class == "PALADIN" and _G.PowerBarColor["HOLY_POWER"] ) then
	POWER.max = _G.HOLY_POWER_FULL;
	if( not mop_500 ) then
		POWER.max = _G.MAX_HOLY_POWER;
	end
	
	POWER.name = "HOLY_POWER";
	POWER.spell = _G.SPELL_POWER_HOLY_POWER;
	POWER.color = _G.PowerBarColor["HOLY_POWER"];
	
	POWER.reqlevel = _G.PALADINPOWERBAR_SHOW_LEVEL;
	
	POWER.color_override[4] = {r = 0.95, g = 0.2, b = 0.2};
	POWER.color_override[5] = POWER.color_override[4];
	
elseif( class == "MONK" and _G.PowerBarColor["CHI"] ) then
	POWER.max = 4;
	POWER.name = "CHI";
	POWER.spell = _G.SPELL_POWER_CHI;
	POWER.color = {r = 0.55, g = 1, b = 0.71}; --_G.PowerBarColor["CHI"]; -- too bright
	
elseif( class == "PRIEST" ) then
	POWER.max = _G.PRIEST_BAR_NUM_ORBS;
	POWER.name = "SHADOW_ORBS";
	POWER.spell = _G.SPELL_POWER_SHADOW_ORBS;
	POWER.color = {r = 0.6, g = 0.2, b = 0.8};
	
	POWER.reqspec = _G.SPEC_PRIEST_SHADOW;
	POWER.reqlevel = _G.SHADOW_ORBS_SHOW_LEVEL;
	POWER.enabled = false;
	
	table.insert(POWER.events, "PLAYER_TALENT_UPDATE");
	
	function module:PLAYER_ENTERING_WORLD()
		self:PLAYER_TALENT_UPDATE();
	end
	
elseif( class == "WARLOCK" and spec == _G.SPEC_WARLOCK_AFFLICTION and _G.PowerBarColor["SOUL_SHARDS"] ) then
	POWER.max = 3;
	POWER.name = "SOUL_SHARDS";
	POWER.spell = _G.SPELL_POWER_SOUL_SHARDS;
	POWER.color = _G.PowerBarColor["SOUL_SHARDS"];
	
	POWER.reqspec = _G.SPEC_WARLOCK_AFFLICTION;
	POWER.enabled = false;
	
	table.insert(POWER.events, "PLAYER_TALENT_UPDATE");
	
	function module:PLAYER_ENTERING_WORLD()
		self:PLAYER_TALENT_UPDATE();
	end
elseif( class == "DEATHKNIGHT" ) then
	POWER.max = 6;
	POWER.name = "RUNES";
	POWER.spell = _G.SPELL_POWER_RUNES;
	POWER.color = {r = 0.8, g = 0.1, b = 1};
	
	POWER.color_override[1] = {r = 0.7, g = 0, b = 0};
	POWER.color_override[2] = POWER.color_override[1];
	POWER.color_override[3] = {r = 0, g = 0.7, b = 0};
	POWER.color_override[4] = POWER.color_override[3];
	POWER.color_override[5] = {r = 0, g = 0.7, b = 0.7};
	POWER.color_override[6] = POWER.color_override[5];
	
	POWER.events = {"PLAYER_ENTERING_WORLD", "RUNE_POWER_UPDATE", "RUNE_TYPE_UPDATE"};
else
	return;
end

if( level <= POWER.reqlevel ) then
	POWER.enabled = false;
	table.insert(POWER.events, "PLAYER_LEVEL_UP");
	
	function module:PLAYER_LEVEL_UP()
		if( level >= POWER.reqlevel ) then
			self:UnregisterEvent("PLAYER_LEVEL_UP");
			update_player(self);
		end
	end
end

local FRAME_SPACE = 2;
local FRAME_BORDER = 0;

function module:UpdateFrame(frame)
	if( frame.unit ~= "player" or not POWER.enabled ) then
		return self:ClearFrame(frame);
	end
	
	local id = self.id;
	local control = frame[id];
	local made_control = not control;
	
	if( made_control ) then
		control = PitBull4.Controls.MakeBetterStatusBar(frame);
		frame[id] = control;
	end
	
	POWER.max = _G.UnitPowerMax("player", POWER.spell);
	POWER.current = _G.UnitPower("player", POWER.spell);
	
	if( control:GetWidth() > 0 ) then
		local width = (control:GetWidth() - ((POWER.max - 1) * FRAME_SPACE)) / POWER.max;
		local half = width / 2;
		local height = control:GetHeight();
		
		local power;
		
		for i = 1, POWER.max do
			if( not control[i] ) then
				control[i] = PitBull4.Controls.MakeInlinePower(control, i);
			end
			power = control[i];
			power:UpdateColor(POWER.color_override[i] or POWER.color);
			
			power:ClearAllPoints();
			power:SetSize(width, height);
			power:SetPoint("CENTER", control, "LEFT", FRAME_BORDER + (i - 1) * (FRAME_SPACE + width) + half, 0);
			
			if( i <= POWER.current ) then
				power:Activate();
			else
				power:Deactivate();
			end
		end
	end
	
	made_control = changed or made_control;
	changed = false;
	
	return made_control;
end

-- dirty hack but death knights behave soooo different
if( POWER.name == "RUNES" ) then
	function module:UpdateFrame(frame)
		if( frame.unit ~= "player" or not POWER.enabled ) then
			return self:ClearFrame(frame);
		end
		
		local id = self.id;
		local control = frame[id];
		local made_control = not control;
		
		if( made_control ) then
			control = PitBull4.Controls.MakeBetterStatusBar(frame);
			frame[id] = control;
		end
		
		if( control:GetWidth() > 0 ) then
			local width = (control:GetWidth() - ((POWER.max - 1) * FRAME_SPACE)) / POWER.max;
			local half = width / 2;
			local height = control:GetHeight();
			
			local power;
			
			for i = 1, POWER.max do
				if( not control[i] ) then
					control[i] = PitBull4.Controls.MakeInlinePower(control, i);
				end
				power = control[i];
				power:UpdateColor(_G.GetRuneType(i) == 4 and POWER.color or POWER.color_override[i]);
				
				power:ClearAllPoints();
				power:SetSize(width, height);
				power:SetPoint("CENTER", control, "LEFT", FRAME_BORDER + (i - 1) * (FRAME_SPACE + width) + half, 0);
				
				local start, duration, ready = _G.GetRuneCooldown(i);
				
				if( ready ) then
					power:Activate();
				else
					power:Deactivate();
				end
			end
		end
		
		made_control = changed or made_control;
		changed = false;
		
		return made_control;
	end
end



function module:GetValue(frame) return 0 end
module.GetExampleValue = module.GetValue;
function module:GetColor(frame, value) return 0, 0, 0, 0 end
module.GetExampleColor = module.GetColor;
